@@ -5,9 +5,10 @@
 
 extern int  _XS_get_verbose(void);
 extern void _XS_set_verbose(int v);
+extern int  _XS_get_callgmp(void);
+extern void _XS_set_callgmp(int v);
 
 extern int _XS_is_prime(UV x);
-extern UV  next_trial_prime(UV x);
 extern UV  _XS_next_prime(UV x);
 extern UV  _XS_prev_prime(UV x);
 
